@@ -1,7 +1,7 @@
 /*
     Implementation of GPTData class derivative with popt-based command
     line processing
-    Copyright (C) 2010-2013 Roderick W. Smith
+    Copyright (C) 2010-2014 Roderick W. Smith
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -63,7 +63,7 @@ void GPTDataCL::LoadBackupFile(string backupFile, int &saveData, int &neverSaveD
 int GPTDataCL::DoOptions(int argc, char* argv[]) {
    GPTData secondDevice;
    int opt, numOptions = 0, saveData = 0, neverSaveData = 0;
-   int partNum = 0, saveNonGPT = 1, retval = 0, pretend = 0;
+   int partNum = 0, newPartNum = -1, saveNonGPT = 1, retval = 0, pretend = 0;
    uint64_t low, high, startSector, endSector, sSize;
    uint64_t temp; // temporary variable; free to use in any case
    char *device;
@@ -93,6 +93,7 @@ int GPTDataCL::DoOptions(int argc, char* argv[]) {
       {"new", 'n', POPT_ARG_STRING, &newPartInfo, 'n', "create new partition", "partnum:start:end"},
       {"largest-new", 'N', POPT_ARG_INT, &largestPartNum, 'N', "create largest possible new partition", "partnum"},
       {"clear", 'o', POPT_ARG_NONE, NULL, 'o', "clear partition table", ""},
+      {"print-mbr", 'O', POPT_ARG_NONE, NULL, 'O', "print MBR partition table", ""},
       {"print", 'p', POPT_ARG_NONE, NULL, 'p', "print partition table", ""},
       {"pretend", 'P', POPT_ARG_NONE, NULL, 'P', "make changes in memory, but don't write them", ""},
       {"transpose", 'r', POPT_ARG_STRING, &twoParts, 'r', "transpose two partitions", "partnum:partnum"},
@@ -122,7 +123,7 @@ int GPTDataCL::DoOptions(int argc, char* argv[]) {
 
    // Do one loop through the options to find the device filename and deal
    // with options that don't require a device filename, to flag destructive
-   // (o, z, or Z) options, and to flag presence of an
+   // (o, z, or Z) options, and to flag presence of a --pretend/-P option
    while ((opt = poptGetNextOpt(poptCon)) > 0) {
       switch (opt) {
          case 'A':
@@ -161,6 +162,8 @@ int GPTDataCL::DoOptions(int argc, char* argv[]) {
                case 'A': {
                   if (cmd != "list") {
                      partNum = (int) GetInt(attributeOperation, 1) - 1;
+                     if (partNum < 0)
+                        partNum = newPartNum;
                      if ((partNum >= 0) && (partNum < (int) GetNumParts())) {
                         switch (ManageAttributes(partNum, GetString(attributeOperation, 2),
                            GetString(attributeOperation, 3))) {
@@ -191,9 +194,14 @@ int GPTDataCL::DoOptions(int argc, char* argv[]) {
                   free(backupFile);
                   break;
                case 'c':
+                  cout << "Setting name!\n";
                   JustLooking(0);
                   partNum = (int) GetInt(partName, 1) - 1;
+                  if (partNum < 0)
+                     partNum = newPartNum;
+                  cout << "partNum is " << partNum << "\n";
                   if ((partNum >= 0) && (partNum < (int) GetNumParts())) {
+                     cout << "REALLY setting name!\n";
                      name = GetString(partName, 2);
                      if (SetName(partNum, (UnicodeString) name.c_str())) {
                         saveData = 1;
@@ -276,19 +284,19 @@ int GPTDataCL::DoOptions(int argc, char* argv[]) {
                   break;
                case 'n':
                   JustLooking(0);
-                  partNum = (int) GetInt(newPartInfo, 1) - 1;
-                  if (partNum < 0)
-                     partNum = FindFirstFreePart();
+                  newPartNum = (int) GetInt(newPartInfo, 1) - 1;
+                  if (newPartNum < 0)
+                     newPartNum = FindFirstFreePart();
                   low = FindFirstInLargest();
                   Align(&low);
                   high = FindLastInFree(low);
                   startSector = IeeeToInt(GetString(newPartInfo, 2), sSize, low, high, low);
                   endSector = IeeeToInt(GetString(newPartInfo, 3), sSize, startSector, high, high);
-                  if (CreatePartition(partNum, startSector, endSector)) {
+                  if (CreatePartition(newPartNum, startSector, endSector)) {
                      saveData = 1;
                   } else {
-                     cerr << "Could not create partition " << partNum + 1 << " from "
-                     << startSector << " to " << endSector << "\n";
+                     cerr << "Could not create partition " << newPartNum + 1 << " from "
+                          << startSector << " to " << endSector << "\n";
                      neverSaveData = 1;
                   } // if/else
                   free(newPartInfo);
@@ -313,6 +321,9 @@ int GPTDataCL::DoOptions(int argc, char* argv[]) {
                   ClearGPTData();
                   saveData = 1;
                   break;
+               case 'O':
+                   DisplayMBRData();
+                   break;
                case 'p':
                   DisplayGPTData();
                   break;
@@ -351,6 +362,8 @@ int GPTDataCL::DoOptions(int argc, char* argv[]) {
                case 't':
                   JustLooking(0);
                   partNum = (int) GetInt(typeCode, 1) - 1;
+                  if (partNum < 0)
+                     partNum = newPartNum;
                   if ((partNum >= 0) && (partNum < (int) GetNumParts())) {
                      typeHelper = GetString(typeCode, 2);
                      if ((typeHelper != (GUIDData) "00000000-0000-0000-0000-000000000000") &&
@@ -373,6 +386,8 @@ int GPTDataCL::DoOptions(int argc, char* argv[]) {
                   JustLooking(0);
                   saveData = 1;
                   partNum = (int) GetInt(partGUID, 1) - 1;
+                  if (partNum < 0)
+                     partNum = newPartNum;
                   if ((partNum >= 0) && (partNum < (int) GetNumParts())) {
                      SetPartitionGUID(partNum, GetString(partGUID, 2).c_str());
                   }
@@ -467,14 +482,14 @@ int GPTDataCL::BuildMBR(char* argument, int isHybrid) {
    int numParts, allOK = 1, i, origPartNum;
    MBRPart newPart;
    BasicMBRData newMBR;
-   
+
    if (argument != NULL) {
       numParts = CountColons(argument) + 1;
       if (numParts <= (4 - isHybrid)) {
          newMBR.SetDisk(GetDisk());
          for (i = 0; i < numParts; i++) {
             origPartNum = GetInt(argument, i + 1) - 1;
-            if (IsUsedPartNum(origPartNum)) {
+            if (IsUsedPartNum(origPartNum) && (partitions[origPartNum].IsSizedForMBR() == MBR_SIZED_GOOD)) {
                newPart.SetInclusion(PRIMARY);
                newPart.SetLocation(operator[](origPartNum).GetFirstLBA(),
                                    operator[](origPartNum).GetLengthLBA());
@@ -482,7 +497,7 @@ int GPTDataCL::BuildMBR(char* argument, int isHybrid) {
                newPart.SetType((uint8_t)(operator[](origPartNum).GetHexType() / 0x0100));
                newMBR.AddPart(i + isHybrid, newPart);
             } else {
-               cerr << "Partition " << origPartNum << " does not exist! Aborting operation!\n";
+               cerr << "Original partition " << origPartNum + 1 << " does not exist or is too big! Aborting operation!\n";
                allOK = 0;
             } // if/else
          } // for
@@ -493,7 +508,8 @@ int GPTDataCL::BuildMBR(char* argument, int isHybrid) {
             newPart.SetType(0xEE);
             newMBR.AddPart(0, newPart);
          } // if
-         SetProtectiveMBR(newMBR);
+         if (allOK)
+            SetProtectiveMBR(newMBR);
       } else allOK = 0;
    } else allOK = 0;
    if (!allOK)
